package com.example.gitserver.service.impl;

import com.example.gitserver.common.ResultData;
import com.example.gitserver.service.GitService;
import com.example.gitserver.utils.GitUtil;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.ListBranchCommand;
import org.eclipse.jgit.api.PullResult;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.TransportException;
import org.eclipse.jgit.lib.Constants;
import org.eclipse.jgit.lib.Ref;
import org.eclipse.jgit.revwalk.RevCommit;
import org.eclipse.jgit.transport.PushResult;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service("gitService")
public class GitServiceImpl implements GitService {

    Logger logger= LoggerFactory.getLogger(GitServiceImpl.class);


    @Override
    public void clone(String remotePath, File localFile, String user, String pwd,String branch) throws GitAPIException {
        UsernamePasswordCredentialsProvider provider = new UsernamePasswordCredentialsProvider(user, pwd);
        try (Git git = Git.cloneRepository()
                .setURI(remotePath)
                .setBranch(branch)
                .setDirectory(localFile)
                .setCredentialsProvider(provider)
                //.setProgressMonitor(new CloneProgressMonitor())
                .call()){
        }
    }


    @Override
    public void pull(String localPath,String branchName,String user,String pwd) throws IOException, GitAPIException {
        try(Git git = GitUtil.openGit(localPath)){
            UsernamePasswordCredentialsProvider provider = new UsernamePasswordCredentialsProvider(user, pwd);
           PullResult result= git.pull().setRemoteBranchName(branchName).setCredentialsProvider(provider).call();
            logger.info(result.isSuccessful()+"...."+result.toString());
        }
    }


    @Override
    public void checkout(String localPath,String branchName) throws IOException, GitAPIException {
        try(Git git = GitUtil.openGit(localPath)){
            git.checkout().setCreateBranch(true).setName(branchName).setForced(true).call();
        }

    }



    @Override
    public List<String> getRemoteBranchList(String localPath) throws GitAPIException, IOException {
        try (Git git = GitUtil.openGit(localPath)) {
            List<Ref> list = git.branchList().setListMode(ListBranchCommand.ListMode.REMOTE).call();
            List<String> all = new ArrayList<>(list.size());
            list.forEach(ref -> {
                String name = ref.getName();
                if (name.startsWith(Constants.R_REMOTES + Constants.DEFAULT_REMOTE_NAME)) {
                    all.add(name.substring((Constants.R_REMOTES + Constants.DEFAULT_REMOTE_NAME).length() + 1));
                }
            });
            return all;
        }
    }

    @Override
    public List<String> getLocalBranchList(String localPath) throws GitAPIException, IOException {
        try (Git git = GitUtil.openGit(localPath)) {
            List<Ref> list = git.branchList().setListMode(ListBranchCommand.ListMode.ALL).call();
            List<String> all = new ArrayList<>(list.size());
            list.forEach(ref -> {
                String name = ref.getName();
                if (name.startsWith(Constants.R_HEADS)) {
                    all.add(name.substring((Constants.R_HEADS).length()));
                }
            });
            return all;
        }
    }

    @Override
    public Ref changeLocalBranch(String localPath,String branch) throws IOException, GitAPIException {
       try (Git git = GitUtil.openGit(localPath)) {
            Ref ref = git.checkout().setCreateBranch(false).setName(branch).call();
            return ref;
       }
    }

    @Override
    public void commit(String localPath,String desc) throws IOException, GitAPIException {
        try (Git git = GitUtil.openGit(localPath)) {
            git.add().addFilepattern(".").call();
            RevCommit  revCommit= git.commit().setAll(true).setMessage(desc).call();
            logger.info(revCommit.getFullMessage()+".."+revCommit.getTree());
        }

    }


    @Override
    public void push(String localPath,String branch,String user,String pwd) throws IOException, GitAPIException {
        try (Git git = GitUtil.openGit(localPath)) {
             UsernamePasswordCredentialsProvider provider = new UsernamePasswordCredentialsProvider(user, pwd);
            // Ref ref=changeLocalBranch(localPath,branch);
            // git.push().add(ref).setCredentialsProvider(provider).call();
            // git.push().setRemote("origin/"+branch)
            //        .setCredentialsProvider(provider).call();
            Iterable<PushResult> list =  git.push().setRemote("origin").add(branch).setCredentialsProvider(provider).call();
            if(list != null) {
                for(PushResult pr : list) {
                    logger.info(pr.getMessages());
                }
            }

        }
    }
}
